classdef dipole % contains positions of dipoles for Measure Projection
    properties
        location = []; % an N x 3 array containing N dipole positions.
        direction = []; % an N x 3 array containing N dipole directions.
        coordinateFormat = 'mni';
    end % properties
    methods
        function plotDipoleDensity(obj, varargin)
            dipoledensity( obj.location, 'coordformat', obj.coordinateFormat, 'plot','on', varargin{:});
        end;
        function plotDipole(obj, withCortex, createNewFigure, varargin)
            % plotDipole(obj, withCortex, createNewFigure, varargin)
            
            if nargin < 2
                withCortex = true;
            end;
            
            if nargin < 3
                createNewFigure = true;
            end;
            
            if createNewFigure
                figure;
            end;
                  
            % only shows 3D spheres for dipoles when their number if less
            % than a certain limit. This is to prevent verylong plot time.
            
            if obj.numberOfDipoles < 600
                plot3dSpheres = 'on';
            else
                plot3dSpheres = 'off';
            end;
            
            plot_dipplot_with_cortex(obj.location, withCortex, 'coordformat', obj.coordinateFormat, 'spheres',plot3dSpheres, 'gui', 'off', varargin{:});
            %dipplot(source, 'coordformat', obj.coordinateFormat, 'spheres',plot3dSpheres 'gui', 'off', varargin{:});
        end;
        function [dipoleId sortedDipoleDensity orderOfDipoles dipoleDenisty dipoleDenistyInRegion] = getDipoleDensityContributionToRegionOfInterest(obj, regionOfInterest, projection, cutoffRatio)
            % [dipoleId sortedDipoleDensity orderOfDipoles dipoleDenistyInRegion dipoleDenistyInRegion] = getDipoleDensityContributionToRegionOfInterest(obj, projection, regionOfInterest, cutoffRatio)
            % projection is a type of meanProjection (or compatible).
            % return dipoles ids (in dipoleId)  which cumulatively contribute at least 'cutoffRatio' to the dipole
            % denisty over the region.
            
            if nargin < 4
                cutoffRatio = 0.97;
            end;
            
            [projectionMatrix dipoleDenisty gaussianWeightMatrix]= pr.meanProjection.getProjectionMatrix(obj, projection.headGrid, projection.standardDeviationOfEstimatedDipoleLocation, projection.numberOfStandardDeviationsToTruncatedGaussaian, regionOfInterest, true);
            
            dipoleDenistyInRegion = sum(gaussianWeightMatrix,2);
            [sortedDipoleDensity orderOfDipoles] = sort(dipoleDenistyInRegion,'descend');
            
            cumulativePercentProjected = cumsum(sortedDipoleDensity) / sum(sortedDipoleDensity);
            numberOfSortedDipolesToReturn= find(cumulativePercentProjected > cutoffRatio,1,  'first');
            
            
            dipoleId = orderOfDipoles(1:numberOfSortedDipolesToReturn);
        end;
        function obj = convertDipoleCoordinatesFromSphericalToMni(obj)                      
            fprintf('converting dipoles from spherical to MNI coordinates...\n');
            for i=1:size(obj.location,1)
                locationInMniCoordinate = sph2spm * [obj.location(i,:) 1]';
                obj.location(i,:) = locationInMniCoordinate(1:3);
                
                directionInMniCoordinate = sph2spm * [obj.direction(i,:) 1]';               
                obj.direction(i,:) = directionInMniCoordinate(1:3);
            end;
        end;
        function theNumber = numberOfDipoles(obj)
            theNumber = size(obj.location, 1);
        end;
    end;
end