classdef scalpmapOfStudy
    % holds scalpmap data and handles plotting, polarity normalization...
    properties
        originalChannelWeight  = []; % a 3D array (comps, scalp x, scalp y) that holds the topomaps (ICA mixing weights for channels) before polarity normalization
        normalizedChannelWeight  = []; % a 3D (comps, scalp x, scalp y) array that holds the topomaps (ICA mixing weights for channels)
        gridX, gridY = []; % the 2D grid on which scalp maps are plotted.
        normalizationPolarity = []; % a vector (containing 1 and -1) that hold polarities for normalizaing scalp maps.
    end % properties
    methods
        function obj = scalpmapOfStudy(STUDY, ALLEEG, icIndexForEachDipole, varargin) % constructor
            % conditionId is an optional input that specifies which conditions should be included.
            % by default all conditions are included.
            
            if nargin > 0
                
                % if icIndexForEachDipole is not provided, assume 1-to-1 association between ICs and
                % Dipoles (as opposed to potentially two dipoles for some ICs).
                
                if nargin < 3
                    icIndexForEachDipole = 1:length(STUDY.cluster(1).comps);
                end;
                
                fprintf('Reading scalp-map data...\n');
                for i = 1:length(STUDY.cluster(1).comps)
                    [channelWeightForIc(i,:,:), obj.gridY, obj.gridX ] = std_readtopo(ALLEEG, STUDY.cluster(1).sets(1, i), STUDY.cluster(1).comps(i), 'mode', '2dmap');
                end;
                
                obj.originalChannelWeight = channelWeightForIc(icIndexForEachDipole,:,:);
                obj = obj.normalizePolarity;
            end;
        end;
        function obj = normalizePolarity(obj) % normalize polarities
            obj.normalizationPolarity = normalize_ic_polarity(obj.originalChannelWeight(:,:));
            obj.normalizedChannelWeight = obj.originalChannelWeight .* repmat(obj.normalizationPolarity', [1, size(obj.originalChannelWeight, 2), size(obj.originalChannelWeight, 3)]);
        end;
        function newObj = createSubsetForId(obj, subsetId)
            newObj = obj;
            newObj.originalChannelWeight = obj.originalChannelWeight(subsetId,:,:);
            newObj = newObj.normalizePolarity;
        end;
        function plot(obj, subsetId, visualizationPolarity, createNewFigure)
            % plot(obj, subsetId, visualizationPolarity, createNewFigure)
            
            if nargin<2 || isempty(subsetId)
                subsetId = 1:size(obj.normalizedChannelWeight, 1);
                
                if length(subsetId) > 100
                    fprintf('There are more than 100 scalp-maps, by default only the first 100 are displayed.\n');
                    subsetId = 1:100;
                end;
            end;
            
            if nargin<3
                visualizationPolarity = 1;
            end;
            
            if nargin<4
                createNewFigure = true;
            end;
            
            numberOfRows = ceil((length(subsetId) * 3/4) .^ 0.5);
            numberOfColumns = ceil(numberOfRows * 4/3);
            
            if createNewFigure
                figure;
            end;
            
            for i=1:length(subsetId)
                sbplot(numberOfRows, numberOfColumns, i);
                toporeplot(visualizationPolarity * squeeze(obj.normalizedChannelWeight(subsetId(i),:,:)),'plotrad',0.5,  'intrad' , 0.5);
            end;
        end
    end;
end