function [subjectBestCorrelation subjectBestCorrelationPvalue]= subject_measure_similarity(STUDY, pos, linearizedProjectedMeasure,  linearizedMeasure, brainIcIds, searchRadius)
% pos is N x 3 matrix

if nargin < 6
    searchRadius = 30; % mm
end;

subjectId = STUDY.cluster(1).sets(1,brainIcIds);
uniqeSubjectId = unique(subjectId);

dipoleLocation = STUDY.preclust.similarity.compPos(brainIcIds,:);

% for each dipole, find the closest distnce to locations in pos (N x 3 matrix)
distanceToDipoles = inf(1, size(dipoleLocation,1));
for i=1:size(pos,1)
    distanceToDipoles = min(distanceToDipoles, sum((dipoleLocation - repmat(pos(i,:), size(dipoleLocation,1), 1))' .^2) .^ 0.5);
end;


% calculate the correlation between IC measures with the projected measure
for i=1:size(linearizedMeasure,2)
    t = corrcoef(linearizedProjectedMeasure, linearizedMeasure(:, i)');
    projectedCorrelation(i) = t(1,2);
end;

numberOfPermutations = 200;
bestCorrelationIcsIdsForAllSubjects = [];

for j=1:length(uniqeSubjectId)
    subjectIcs = find(subjectId == uniqeSubjectId(j));
    
    [subjectBestCorrelation(j) subjectBestCorrelationIcId] = bestCorrelationForIcSet(subjectIcs);
    
    if ~isnan(subjectBestCorrelationIcId)
        bestCorrelationIcsIdsForAllSubjects = [bestCorrelationIcsIdsForAllSubjects subjectBestCorrelationIcId];
    end;
end;

if nargout > 1
    
    % do not include best subject ICs in the permutation
    icsForPermutation = setdiff(1:length(subjectId), bestCorrelationIcsIdsForAllSubjects);
    
    for j=1:length(uniqeSubjectId)
        subjectIcs = find(subjectId == uniqeSubjectId(j));
        numberOfSubjectIcs = numel(subjectIcs);
        
        for permutationNumber = 1:numberOfPermutations
            surrogateIds = icsForPermutation(randi(length(icsForPermutation),1, numberOfSubjectIcs));
            surrogateBestCorrelation(permutationNumber) = bestCorrelationForIcSet(surrogateIds);
        end;
        
        
        subjectBestCorrelationPvalue(j) = sum(surrogateBestCorrelation > subjectBestCorrelation(j)) / numberOfPermutations;
    end;
end;

subjectBestCorrelation(isinf(subjectBestCorrelation)) = nan;

    function [bestCorrelation bestCorrelationIcId] = bestCorrelationForIcSet(setIds)
        setIcsInProximity = setIds(distanceToDipoles(setIds) < searchRadius);
        
        bestCorrelation = -Inf;
        bestCorrelationIcId = nan; % means no IC
        
        if ~isempty(setIcsInProximity)
            for k=1:length(setIcsInProximity)
                if projectedCorrelation(setIcsInProximity(k)) > bestCorrelation
                    bestCorrelation = projectedCorrelation(setIcsInProximity(k));
                    bestCorrelationIcId = setIcsInProximity(k);
                end;
            end;
        end;
    end
end

